@@ -10,9 +10,9 @@ import re
 from mopidy.config import keyring
 from mopidy.config.schemas import *  # noqa
 from mopidy.config.types import *  # noqa
-from mopidy.utils import path
+from mopidy.utils import path, versioning
 
-logger = logging.getLogger('mopidy.config')
+logger = logging.getLogger(__name__)
 
 _logging_schema = ConfigSchema('logging')
 _logging_schema['console_format'] = String()
@@ -25,6 +25,7 @@ _loglevels_schema = LogLevelConfigSchema('loglevels')
 _audio_schema = ConfigSchema('audio')
 _audio_schema['mixer'] = String()
 _audio_schema['mixer_track'] = String(optional=True)
+_audio_schema['mixer_volume'] = Integer(optional=True, minimum=0, maximum=100)
 _audio_schema['output'] = String()
 _audio_schema['visualizer'] = String(optional=True)
 
@@ -41,6 +42,18 @@ _proxy_schema['password'] = Secret(optional=True)
 
 _schemas = [_logging_schema, _loglevels_schema, _audio_schema, _proxy_schema]
 
+_INITIAL_HELP = """
+# For further information about options in this file see:
+#   http://docs.mopidy.com/
+#
+# The initial commented out values reflect the defaults as of:
+#   %(versions)s
+#
+# Available options and defaults might have changed since then,
+# run `mopidy config` to see the current effective config and
+# `mopidy --version` to check the current version.
+"""
+
 
 def read(config_file):
     """Helper to load config defaults in same way across core and extensions"""
@@ -66,7 +79,25 @@ def format(config, extensions, comments=None, display=True):
     # need to know about extensions.
     schemas = _schemas[:]
     schemas.extend(e.get_config_schema() for e in extensions)
-    return _format(config, comments or {}, schemas, display)
+    return _format(config, comments or {}, schemas, display, False)
+
+
+def format_initial(extensions):
+    config_dir = os.path.dirname(__file__)
+    defaults = [read(os.path.join(config_dir, 'default.conf'))]
+    defaults.extend(e.get_default_config() for e in extensions)
+    raw_config = _load([], defaults, [])
+
+    schemas = _schemas[:]
+    schemas.extend(e.get_config_schema() for e in extensions)
+
+    config, errors = _validate(raw_config, schemas)
+
+    versions = ['Mopidy %s' % versioning.get_version()]
+    for extension in sorted(extensions, key=lambda ext: ext.dist_name):
+        versions.append('%s %s' % (extension.dist_name, extension.version))
+    description = _INITIAL_HELP.strip() % {'versions': '\n#   '.join(versions)}
+    return description + '\n\n' + _format(config, {}, schemas, False, True)
 
 
 def _load(files, defaults, overrides):
@@ -89,8 +120,8 @@ def _load(files, defaults, overrides):
             with io.open(filename, 'rb') as filehandle:
                 parser.readfp(filehandle)
         except configparser.MissingSectionHeaderError as e:
-            logging.warning('%s does not have a config section, not loaded.',
-                            filename)
+            logger.warning('%s does not have a config section, not loaded.',
+                           filename)
         except configparser.ParsingError as e:
             linenos = ', '.join(str(lineno) for lineno, line in e.errors)
             logger.warning(
@@ -128,7 +159,7 @@ def _validate(raw_config, schemas):
     return config, errors
 
 
-def _format(config, comments, schemas, display):
+def _format(config, comments, schemas, display, disable):
     output = []
     for schema in schemas:
         serialized = schema.serialize(
@@ -137,14 +168,18 @@ def _format(config, comments, schemas, display):
             continue
         output.append(b'[%s]' % bytes(schema.name))
         for key, value in serialized.items():
+            if isinstance(value, types.DeprecatedValue):
+                continue
             comment = bytes(comments.get(schema.name, {}).get(key, ''))
             output.append(b'%s =' % bytes(key))
             if value is not None:
                 output[-1] += b' ' + value
             if comment:
-                output[-1] += b'  # ' + comment.capitalize()
+                output[-1] += b'  ; ' + comment.capitalize()
+            if disable:
+                output[-1] = re.sub(r'^', b'#', output[-1], flags=re.M)
         output.append(b'')
-    return b'\n'.join(output)
+    return b'\n'.join(output).strip()
 
 
 def _preprocess(config_string):
